@@ -4,7 +4,7 @@ use crate::utils::coord::{Coord, FromIndex};
 use crate::utils::draw::Draw;
 use crate::utils::figure::Figure;
 use crate::utils::piece::Piece;
-use std::collections::HashSet;
+use std::collections::{HashMap, HashSet};
 use std::ops::Range;
 
 // Define types for improved readability.
@@ -50,6 +50,9 @@ pub struct Game {
 
     /// Current state of the full-move clock.
     pub full_move_clock: u16,
+
+    /// UCI Notation of the move that has been played
+    pub uci: String,
 }
 
 impl Game {
@@ -74,10 +77,11 @@ impl Game {
             en_passant: None,
             half_move_clock: 0,
             full_move_clock: 1,
+            uci: "0000".to_string(),
         };
     }
 
-    pub fn to_fen(self) -> String {
+    pub fn to_fen_list(self) -> [String; 6] {
         return [
             position_to_fen(self.position),
             self.color.to_string(),
@@ -88,8 +92,31 @@ impl Game {
             },
             self.half_move_clock.to_string(),
             self.full_move_clock.to_string(),
+        ];
+    }
+
+    pub fn to_fen_map(self) -> HashMap<String, String> {
+        let keys: [String; 6] = [
+            "FEN",
+            "Color",
+            "Castling",
+            "EnPassant",
+            "HalfMoveClock",
+            "FullMoveClock",
         ]
-        .join(" ");
+        .map(|x| x.to_string());
+
+        let values = self.to_fen_list();
+
+        return HashMap::from_iter(
+            keys.into_iter()
+                .enumerate()
+                .map(|(k, key)| (key, values[k].clone())),
+        );
+    }
+
+    pub fn to_fen(self) -> String {
+        return self.to_fen_list().join(" ");
     }
 
     pub fn play_move(self, mv: String) -> Self {
@@ -168,6 +195,15 @@ impl Game {
                 }
             }
 
+            // Design UCI representation of a move.
+            let mut uci: String = "".to_string();
+            uci.push_str(&moving_figure.coord.to_string()[..]);
+            uci.push_str(&draw.target.to_string()[..]);
+            if draw.is_promo {
+                // uci is always lowercase, thus use lowercase char induced by black.
+                uci.push(draw.promoted_piece.unwrap().to_char(Color::B));
+            }
+
             Game {
                 board: self.board,
                 position,
@@ -184,6 +220,7 @@ impl Game {
                     Color::B => self.full_move_clock + 1,
                     Color::W => self.full_move_clock,
                 },
+                uci,
             }
         }
     }
@@ -191,6 +228,7 @@ impl Game {
     fn castle(self, mv: String) -> Game {
         let mut figures: FigSet = self.figures.clone();
         let mut position: OptFigures = self.position.clone();
+        let mut uci: String = "".to_string();
 
         // prepare indexes with
         let king_src: usize;
@@ -205,10 +243,12 @@ impl Game {
                 rook_src = 0;
                 king_tgt = 2;
                 rook_tgt = 3;
+                uci = "e8c8".to_string();
             } else {
                 rook_tgt = 5;
                 king_tgt = 6;
                 rook_src = 7;
+                uci = "e8g8".to_string();
             }
         } else {
             king_src = 60;
@@ -216,10 +256,12 @@ impl Game {
                 rook_src = 56;
                 king_tgt = 58;
                 rook_tgt = 59;
+                uci = "e1c1".to_string();
             } else {
                 rook_tgt = 61;
                 king_tgt = 62;
                 rook_src = 63;
+                uci = "e1g1".to_string();
             }
         }
 
@@ -253,6 +295,7 @@ impl Game {
                 Color::W => self.full_move_clock,
                 Color::B => self.full_move_clock + 1,
             },
+            uci,
         }
     }
 
@@ -282,6 +325,7 @@ impl Game {
             en_passant: self.en_passant,
             half_move_clock: self.half_move_clock,
             full_move_clock: self.full_move_clock,
+            uci: self.uci,
         }
     }
 
@@ -306,6 +350,7 @@ impl Game {
             en_passant: self.en_passant,
             half_move_clock: self.half_move_clock,
             full_move_clock: self.full_move_clock,
+            uci: self.uci,
         }
     }
 }
@@ -343,6 +388,9 @@ impl From<String> for Game {
         let half_move_clock = hmc_str.parse::<u16>().unwrap();
         let full_move_clock = fmc_str.parse::<u16>().unwrap();
 
+        // As the fen does not reveal the Move, set null move.
+        let uci = "0000".to_string();
+
         Game {
             board,
             position,
@@ -352,6 +400,7 @@ impl From<String> for Game {
             en_passant,
             half_move_clock,
             full_move_clock,
+            uci,
         }
     }
 }
@@ -953,6 +1002,27 @@ fn check_castling() {
         g2.figures,
         HashSet::from_iter(["Kc1", "Rd1", "rf8", "kg8"].map(|x| Figure::from(x)))
     );
+
+    assert_eq!(g2.uci, "e8g8".to_string());
+}
+
+#[test]
+fn check_fen_map() {
+    let game = Game::from(
+        "rnbqk2r/pppp1ppp/3b1n2/8/1PPPp3/P1N1P3/5PPP/R1BQKBNR b KQkq d3 0 6".to_string(),
+    );
+
+    let fen_map = game.to_fen_map();
+
+    assert_eq!(
+        fen_map["FEN"],
+        "rnbqk2r/pppp1ppp/3b1n2/8/1PPPp3/P1N1P3/5PPP/R1BQKBNR"
+    );
+    assert_eq!(fen_map["Color"], "b");
+    assert_eq!(fen_map["Castling"], "KQkq");
+    assert_eq!(fen_map["EnPassant"], "d3");
+    assert_eq!(fen_map["HalfMoveClock"], "0");
+    assert_eq!(fen_map["FullMoveClock"], "6");
 }
 
 #[test]
@@ -1054,6 +1124,7 @@ fn check_playing_games_pt1() {
         game = game.play_move(mv);
     }
 
+    assert_eq!(game.clone().uci, "c7b6".to_string());
     assert_eq!(
         game.to_fen(),
         "8/7p/1k6/3Q4/2Q5/6Pp/5P1K/8 w - - 1 62".to_string()
@@ -1088,6 +1159,7 @@ fn check_playing_games_pt2() {
         game = game.play_move(mv);
     }
 
+    assert_eq!(game.clone().uci, "g6h6".to_string());
     assert_eq!(game.to_fen(), "7k/5K2/7R/8/8/8/8/8 b - - 60 95".to_string())
 }
 
@@ -1116,10 +1188,11 @@ fn check_playing_games_pt3() {
         game = game.play_move(mv);
     }
 
+    assert_eq!(game.clone().uci, "c1c6".to_string());
     assert_eq!(
         game.to_fen(),
         "8/8/K1q5/8/k2b4/8/8/8 w - - 10 82".to_string()
-    )
+    );
 }
 
 /// https://lichess.org/9opx3qh7
@@ -1142,6 +1215,7 @@ fn check_playing_games_pt4() {
         game = game.play_move(mv);
     }
 
+    assert_eq!(game.clone().uci, "b3b2".to_string());
     assert_eq!(
         game.to_fen(),
         "8/5pk1/6b1/8/8/Q4P2/1pq3PK/8 w - - 0 46".to_string()
@@ -1166,6 +1240,7 @@ fn check_playing_games_pt5() {
         game = game.play_move(mv);
     }
 
+    assert_eq!(game.clone().uci, "c1c2".to_string());
     assert_eq!(
         game.to_fen(),
         "8/p6k/5R2/6p1/1PpPb1Pp/2P2P1P/P1r3K1/4r3 w - - 1 38".to_string()
